;+
;
; @param MODEL
;
; @param FILE_NAME {in}{required}{type=string}
;
; @param NCDF_DB {in}{required}{type=string}
; <location>:<path> or just <path>
;
; @param VAR_NAME {in}{required}{type=string}
; variable name
;
; @keyword ALL_DATA
;
; @uses
; <pro>common</pro>
; <propost_it>com_eg</propost_it>
;
; @version
; $Id$
;
;-
PRO mesh_from_file, model, file_name, ncdf_db, var_name $
    , ALL_DATA=all_data
;
  compile_opt idl2, strictarrsubs
;
@common
@com_eg
;
   IF debug_w THEN BEGIN
    info = report('enter ...')
    IF keyword_set(all_data) THEN BEGIN
     print, '        All_data = ', all_data
    ENDIF
   ENDIF
;
; init grid, sf from file_name
; masks made in data_read
;
   IF strpos(ncdf_db, ':') GE 1 THEN directory = (strsplit(ncdf_db, ':', /EXTRACT))[1] $
    ELSE directory = ncdf_db

   s_file = directory+file_name

   print,'    Model inits for ', model, ' from file: ', s_file

   sm_file = hom_idl+'grids/grids_'+model+'.nc'
   res = find(sm_file)
   varexpp = varexp
   IF res NE 'NOT FOUND' THEN BEGIN

      IF keyword_set(ALL_DATA) THEN BEGIN
         initncdf, s_file, GLAMBOUNDARY = glamboundary_box, PLAIN = 1, fullcgrid = 1
      ENDIF ELSE initncdf, s_file, GLAMBOUNDARY = glamboundary_box, fullcgrid = 1

      print, '    Found mask from ',sm_file

      tmask = byte(read_ncdf('sftlf', 0, 0, /timestep, file = sm_file, /nostruct))

      idx = where(tmask EQ valmask)

      IF idx[0] NE -1 THEN tmask[idx] = 0.
      idx = where(tmask LE 50.)
      tmask[idx] = 0.
      tmask = tmask < 1
      tmask = 1-tmask
      triangles=triangule()

   ENDIF ELSE BEGIN
      CASE var_name OF
         '@@voenergy': BEGIN
            CASE cmd1_back.grid OF
               'T': var_local = 'votemper'
               'T05': var_local = 'votemper'
               'T05#': var_local = 'votemper'
               ELSE: var_local = 'so'
            ENDCASE
            END
         ELSE: var_local = var_read_grd_file
      ENDCASE
      IF strpos(var_local, '(next)') NE -1 THEN BEGIN
         idx = strpos(var_local, '=')
         var_local = strmid(var_local, 0, idx)
         sl_pos = strpos(var_local, '/')
         IF sl_pos NE -1 THEN var_local = strmid(var_local, 0, sl_pos)
      ENDIF

      IF debug_w THEN print, '   var_local = ', var_local

      ; find valmask
      cdfidl=ncdf_open(s_file)
      contient=ncdf_inquire(cdfidl)
      varidl = ncdf_varid(cdfidl, var_local)
      varcontient=ncdf_varinq(cdfidl, var_local)
      valmask = 1.e20
      FOR i = 0, varcontient.natts-1 DO BEGIN
         att_txt = ncdf_attname(cdfidl, varidl, i)
         IF att_txt EQ 'missing_value' OR att_txt EQ 'mask value' OR att_txt EQ '_FillValue' THEN BEGIN
            ncdf_attget, cdfidl, varidl, att_txt, valmask
            IF debug_w THEN print, '     valmask found = ',valmask, ' in attribute ', att_txt
         ENDIF
      ENDFOR
      ncdf_close, cdfidl
      ; build grid
      CASE mesh_type of
         'atm':  BEGIN
            IF keyword_set(ALL_DATA) THEN BEGIN
               initncdf, s_file, GLAMBOUNDARY = glamboundary_box, ZAXISNAME = name_level, PLAIN = 1, fullcgrid = 1
            ENDIF ELSE initncdf, s_file, GLAMBOUNDARY = glamboundary_box, ZAXISNAME = name_level, fullcgrid = 1
         END
         ELSE: BEGIN
            IF keyword_set(ALL_DATA) THEN BEGIN
               initncdf, s_file, USEASMASK = var_local, missing_value = valmask, GLAMBOUNDARY = glamboundary_box, ZAXISNAME = 'z_below_msl', PLAIN = 1, fullcgrid = 1
            ENDIF ELSE initncdf, s_file, USEASMASK = var_local, missing_value = valmask, GLAMBOUNDARY = glamboundary_box, ZAXISNAME = 'z_below_msl', fullcgrid = 1
         END
      ENDCASE
      IF debug_w THEN print, '     Grid info after initncdf :'
      IF debug_w THEN print, '     glamt :', size(glamt)
      IF debug_w THEN print, '     gphit :', size(gphit)
      IF debug_w THEN print, '     gdept :', size(gdept)

   ENDELSE

   varexp = varexpp

   key_offset = [0, 0, 0]

; indice i pour grille j moyenne zonale
;
   diaznl_idx = 1
   IF debug_w THEN BEGIN
    print, '     model, vargrid, varexp = ',model, ' ', vargrid,' ', varexp
    info = report('leaving...')
   ENDIF

END

